import { query } from '/public/vendor/bitecs/index.js'
import { Transform, Camera, Renderable } from '/src/components/index.js'

const vertSource = `#version 300 es
in vec2 a_position;
in vec4 a_color;

uniform vec2 u_resolution;
uniform vec2 u_cameraPosition;
uniform float u_cameraZoom;

out vec4 v_color;

void main() {
  vec2 position = (a_position - u_cameraPosition) * u_cameraZoom;
  vec2 clipSpace = position / (u_resolution / 2.0) - 1.0;
  gl_Position = vec4(clipSpace * vec2(1, -1), 0, 1);
  v_color = a_color;
}`

const fragSource = `#version 300 es
precision mediump float;
in vec4 v_color;
out vec4 outColor;

void main() {
  outColor = v_color;
}`

const createMesh = (world, gl, entity, positions, indices, drawMode) => {
    addComponent(world, Mesh, entity)
    addComponent(world, VertexPositions, entity)

    VertexPositions.data.set(positions, entity * positions.length)

    if (indices) {
        addComponent(world, Indices, entity)
        Indices.data.set(indices, entity * indices.length)
    }

    const vao = gl.createVertexArray()
    gl.bindVertexArray(vao)

    const positionBuffer = gl.createBuffer()
    gl.bindBuffer(gl.ARRAY_BUFFER, positionBuffer)
    gl.bufferData(gl.ARRAY_BUFFER, positions, gl.STATIC_DRAW)
    gl.vertexAttribPointer(0, 3, gl.FLOAT, false, 0, 0)
    gl.enableVertexAttribArray(0)

    if (indices) {
        const indexBuffer = gl.createBuffer()
        gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER, indexBuffer)
        gl.bufferData(gl.ELEMENT_ARRAY_BUFFER, indices, gl.STATIC_DRAW)
    }

    gl.bindVertexArray(null)

    MeshHandle.positionBuffer[entity] = positionBuffer
    MeshHandle.indexBuffer[entity] = indexBuffer
    MeshHandle.vao[entity] = vao
    MeshHandle.drawMode[entity] = drawMode
    MeshHandle.vertexCount[entity] = positions.length / 3
    MeshHandle.indexCount[entity] = indices ? indices.length : 0
}

export const Renderer2D = canvas => {
    const gl = canvas.getContext('webgl2')
    const program = Program.fromSource(gl, vertSource, fragSource)
    const position = new Attribute(gl, program, 'a_position', new Buffer(), new BufferSchema(DataType.f32, { size: 2 }))
    const color = new Attribute(gl, program, 'a_color', new Buffer(), new BufferSchema(DataType.f32, { size: 4 }))
    const resolution = new Uniform(gl, program, 'u_resolution', 2, UniformType.Float)
    const cameraPosition = new Uniform(gl, program, 'u_cameraPosition', 2, UniformType.Float)

    // TODO: make this a property of camera
    const viewport = new Viewport(gl)
    viewport.resize()

    // TODO: separate into window class
    window.addEventListener('resize', () => {
        canvas.width = window.innerWidth
        canvas.height = window.innerHeight
        resolution.set(canvas.width, canvas.height)
        viewport.resize()
    })

    return world => {
        viewport.clear()

        const cameras = query(world, [Transform, Camera])

        // FIXME: Supporting only one camera for now
        const camera = cameras[0]
        const vx = Camera.viewport.position.x[camera]
        const vy = Camera.viewport.position.y[camera]
        const target = Camera.renderTarget[camera] || ctx

        cameraPosition.set(vx, vy)

        const renderable = query(world, [Renderable, Transform])

        for (let i = 0; i < renderable.length; i++) {
            const eid = renderable[i]
            const x = Transform.translation.x[eid] - vx
            const y = Transform.translation.y[eid] - vy

            //target.putImageData(new ImageData(Sprite.data[eid], Sprite.width[eid], Sprite.height[eid]), x, y)
        }
    }
}

